﻿using System.Transactions;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DataModel
{
  [TestClass]
  public class SupplierTest
  {
    [TestMethod]
    public void HomePage_AcceptsValidUrl()
    {
      using (new TransactionScope())
      using (var context = new NorthwindEntities())
      using (new GenericPrincipalScope("Mike", new[] { "Manager" }))
      {
        var supplier = new Supplier();
        supplier.CompanyName = "Test Supplier";
        supplier.HomePage = "http://www.contoso.com";
        context.Suppliers.AddObject(supplier);
        context.SaveChanges();
      }
    }

    [TestMethod]
    [ExpectedValidationException(MemberName = "HomePage")]
    public void HomePage_RejectsInvalidUrl()
    {
      using (new TransactionScope())
      using (var context = new NorthwindEntities())
      using (new GenericPrincipalScope("Mike", new[] { "Manager" }))
      {
        var supplier = new Supplier();
        supplier.CompanyName = "Test Supplier";
        supplier.HomePage = "invalid://url";
        context.Suppliers.AddObject(supplier);
        context.SaveChanges();
      }
    }

    [TestMethod]
    [ExpectedValidationException]
    public void DeletionFailsWhenProductsExist()
    {
      using (new TransactionScope())
      using (var context = new NorthwindEntities())
      using (new GenericPrincipalScope("Mike", new[] { "Manager" }))
      {
        foreach (var supplier in context.Suppliers)
          context.Suppliers.DeleteObject(supplier);
        context.SaveChanges();
      }
    }
  }
}
