﻿namespace DataModel
{
  using System.ComponentModel.DataAnnotations;
  using System.Linq;
  using System.Security.Principal;
  using System.Threading;
  using Unleashed.DataAnnotations;

  [MetadataType(typeof(Metadata))]
  partial class Customer
  {
    public static IQueryable GetQuery(IQueryable source)
    {
      IPrincipal user = Thread.CurrentPrincipal;
      if (user.IsInRole("Customer"))
        source = source.Cast<Customer>().Where(customer => customer.EmailAddress == user.Identity.Name);
      return source;
    }

    [Authorization(
      Allow.Roles, "Manager",
      Allow.Roles, "Employee", Actions.Read | Actions.Update | Actions.Insert,
      Allow.Roles, "Customer", Actions.Read,
      Deny.Users, "*")]
    [CustomQuery(typeof(Customer), "GetQuery")]
    private abstract class Metadata
    {
      [FilterUIHint("Text")]
      public object City { get; set; }

      [Display(Name = "Name")]
      public object ContactName { get; set; }

      [FilterUIHint("Text")]
      public object ContactTitle { get; set; }

      [ScaffoldColumn(false)]
      public object CustomerDemographics { get; set; }

      [Display(Name = "ID")]
      public object CustomerID { get; set; }

      [Display(Name = "State")]
      public object Region { get; set; }

      [Display(Name = "Zip")]
      public object PostalCode { get; set; }
    }
  }
}
