﻿using System;
using System.ComponentModel.DataAnnotations;
using Unleashed.DataAnnotations;

namespace DataModel
{
  [MetadataType(typeof(Metadata))]
  partial class Employee
  {
    [Authorization(
      Allow.Roles, "Manager",
      Allow.Roles, "Employee", Actions.Read,
      Deny.Users, "*")]
    [EntityUIHint("TwoColumn")]
    private abstract class Metadata
    {        
      // Left ---------------------------------------------

      [Display(Name = "First Name", GroupName = "Left", Order = 100)]
      public object FirstName { get; set; }

      [Display(Name = "Last Name", GroupName = "Left", Order = 110)]
      public object LastName { get; set; }

      [Display(Name = "Employee Type", GroupName = "Left", Order = 120)]
      [EnumDataType(typeof(EmployeeType))]
      public object EmployeeType { get; set; }

      [Display(GroupName = "Left", Order = 130)]
      public object Title { get; set; }

      [Display(GroupName = "Left", Order = 140)]
      public object Extension { get; set; }

      [Display(Name = "Email Address", GroupName = "Left", Order = 150)]
      [DataType(DataType.EmailAddress)]
      public object EmailAddress { get; set; }

      [DataType(DataType.Date)]
      [Display(Name = "Hire Date", GroupName = "Left", Order = 160)]
      public object HireDate { get; set; }

      [Display(GroupName = "Left", Order = 170)]
      public object Manager { get; set; }

      [Display(Name = "Direct Reports", GroupName = "Left", Order = 180)]
      public object DirectReports { get; set; }

      [Display(GroupName = "Left", Order = 190)]
      public object Territories { get; set; }

      // Right --------------------------------------------

      [Display(GroupName = "Right", Order = 200)]
      [UIHint("Text", null, "Columns", 65)]
      public object Address { get; set; }

      [Display(GroupName = "Right", Order = 210)]
      public object City { get; set; }

      [Display(Name = "State", GroupName = "Right", Order = 220)]
      [UIHint("Region", "WebForms", "CountryField", "Country")]
      public object Region { get; set; }

      [Display(Name = "Zip", GroupName = "Right", Order = 230)]
      public object PostalCode { get; set; }

      [Display(GroupName = "Right", Order = 240)]
      public object Country { get; set; }

      [DataType(DataType.PhoneNumber)]
      [Required]
      [RegularExpression(@"\(\d{2,3}\)\d{3}-\d{4}",
        ErrorMessage = "The Home Phone must be a valid phone number.")]
      [Display(Name = "Home Phone", ShortName = "Home #", Prompt = "Home Phone Number:", Description = "Please enter a 10-digit Phone Number", GroupName = "Right", Order = 250)]
      public object HomePhone { get; set; }
      
      [DataType(DataType.Date)]
      [Range(typeof(DateTime), "1/1/1900", "12/31/2050")]
      [Display(Name = "Birth Date", GroupName = "Right", Order = 260)]
      public object BirthDate { get; set; }

      [DataType(DataType.Url)]
      [Display(Name = "Photo", GroupName = "Right", Order = 270)]
      public object PhotoPath { get; set; }

      [Display(GroupName = "Right", Order = 280)]
      public object Notes { get; set; }

      // Hidden by default

      [Display(AutoGenerateField = false)]
      public object TitleOfCourtesy { get; set; }

      [Display(AutoGenerateField = false)]
      public object Orders { get; set; }
    }
  }
}