﻿namespace Unleashed.DynamicData.Tests
{
  using System;
  using System.Collections.Generic;
  using System.Collections.ObjectModel;
  using System.Linq;
  using System.Web.DynamicData.ModelProviders;

  /// <summary>
  /// Wraps a given <see cref="DataModelProvider"/> and allows selectively replacing method and property implementations.
  /// </summary>
  class FakeDataModelProvider : DataModelProvider
  {
    public readonly DataModelProvider Original;

    public FakeDataModelProvider(DataModelProvider original)
    {
      this.Original = original;
    }

    public Func<object> CreateContextMethod;
    public override object CreateContext()
    {
      return this.CreateContextMethod != null ? this.CreateContextMethod() : this.Original.CreateContext();
    }

    public Func<IEnumerable<TableProvider>> TablesGetter;
    public override ReadOnlyCollection<TableProvider> Tables
    {
      get { return this.TablesGetter != null ? new ReadOnlyCollection<TableProvider>(this.TablesGetter().ToList()) : this.Original.Tables; }
    }

    public override Type ContextType
    {
      get { return this.Original != null ? this.Original.ContextType : base.ContextType ?? typeof(object); }
      protected set { throw new NotSupportedException(); }
    }

    public override string ToString()
    {
      return this.Original.ToString();
    }

    public override bool Equals(object obj)
    {
      return this.Original.Equals(obj);
    }

    public override int GetHashCode()
    {
      return this.Original.GetHashCode();
    }
  }
}
