﻿namespace Unleashed.DynamicData.Tests
{
  using System;
  using System.Collections.Generic;
  using System.Collections.ObjectModel;
  using System.ComponentModel;
  using System.Linq;
  using System.Security.Principal;
  using System.Web.DynamicData.ModelProviders;

  /// <summary>
  /// Wraps a given <see cref="TableProvider"/> and allows selectively replacing method and property implementations.
  /// </summary>
  internal class FakeTableProvider: TableProvider
  {
    public readonly TableProvider Original;

    public FakeTableProvider(DataModelProvider modelProvider, TableProvider original): base(modelProvider)
    {
      this.Original = original;
    }

    public override string ToString()
    {
      return this.Original.ToString();
    }

    public override ICustomTypeDescriptor GetTypeDescriptor()
    {
      return this.Original != null ? this.Original.GetTypeDescriptor() : base.GetTypeDescriptor();
    }

    public Func<object, IQueryable> GetQueryMethod;
    public override IQueryable GetQuery(object context)
    {
      if (this.GetQueryMethod != null)
        return this.GetQueryMethod(context);
      return this.Original.GetQuery(context);
    }

    public override object EvaluateForeignKey(object row, string foreignKeyName)
    {
      return this.Original.EvaluateForeignKey(row, foreignKeyName);
    }

    public override bool CanDelete(IPrincipal principal)
    {
      return this.Original.CanDelete(principal);
    }

    public override bool CanInsert(IPrincipal principal)
    {
      return this.Original.CanInsert(principal);
    }

    public Func<IPrincipal, bool> CanReadMethod;
    public override bool CanRead(IPrincipal principal)
    {
      if (this.CanReadMethod != null) return this.CanReadMethod(principal);
      if (this.Original != null) return this.Original.CanRead(principal);
      return base.CanRead(principal);
    }

    public override bool CanUpdate(IPrincipal principal)
    {
      return this.Original.CanUpdate(principal);
    }

    public override AttributeCollection Attributes
    {
      get { return this.Original != null ? this.Original.Attributes : base.Attributes; }
    }

    public Func<string> NameGetter;
    public override string Name
    {
      get { return this.NameGetter != null ? this.NameGetter() : this.Original.Name; }
      protected set { throw new NotSupportedException(); }
    }

    public override Type EntityType
    {
      get { return this.Original != null ? this.Original.EntityType : base.EntityType ?? typeof(object); }
      protected set { throw new NotSupportedException(); }
    }

    public Func<IEnumerable<ColumnProvider>> ColumnsGetter;
    public override ReadOnlyCollection<ColumnProvider> Columns
    {
      get
      {
        if (this.ColumnsGetter != null) return new ReadOnlyCollection<ColumnProvider>(this.ColumnsGetter().ToList());
        if (this.Original != null) return this.Original.Columns;
        return new ReadOnlyCollection<ColumnProvider>(new ColumnProvider[0]);
      }
    }

    public override Type ParentEntityType
    {
      get { return this.Original.ParentEntityType; }
      protected set { throw new NotSupportedException();  }
    }

    public override Type RootEntityType
    {
      get { return this.Original.RootEntityType; }
      protected set { base.RootEntityType = value; }
    }

    public override string DataContextPropertyName
    {
      get { return this.Original.DataContextPropertyName; }
      protected set { throw new NotSupportedException(); }
    }

    public override bool Equals(object obj)
    {
      return this.Original.Equals(obj);
    }

    public override int GetHashCode()
    {
      return this.Original.GetHashCode();
    }
  }
}
