namespace Unleashed.DynamicData
{
  using System;
  using System.Collections.Generic;
  using System.ComponentModel;
  using System.ComponentModel.DataAnnotations;
  using System.Linq;
  using System.Web.DynamicData.ModelProviders;
  using DataAnnotations;

  public sealed class UnleashedColumnProvider : ColumnProvider
  {
    public UnleashedColumnProvider(UnleashedTableProvider table, PropertyDescriptor descriptor)
      : base(table)
    {
      this.Name = descriptor.Name;
      this.ColumnType = descriptor.PropertyType;
      this.Nullable = descriptor.PropertyType.IsValueType == false || System.Nullable.GetUnderlyingType(descriptor.PropertyType) != null;
      this.IsReadOnly = descriptor.IsReadOnly;
      this.IsSortable = descriptor.Attributes.OfType<CustomPropertyAttribute>().Any(a => !string.IsNullOrEmpty(a.Expression));
      this.IsCustomProperty = true;
    }

    public override AttributeCollection Attributes
    {
      get
      {
        List<Attribute> attributes = base.Attributes.Cast<Attribute>().ToList();
        var customProperty = attributes.OfType<CustomPropertyAttribute>().FirstOrDefault();
        if (customProperty != null && customProperty.Expression.Contains('.'))
        {
          string[] fields = customProperty.Expression.Split('.');
          string foreignKeyField = fields[0];
          string displayField = fields[1];
          if (!attributes.OfType<UIHintAttribute>().Any())
            attributes.Add(new UIHintAttribute("Parent", null, "ForeignKeyField", foreignKeyField, "DisplayField", displayField));
          if (!attributes.OfType<FilterUIHintAttribute>().Any())
            attributes.Add(new FilterUIHintAttribute("Parent", null, "ForeignKeyField", foreignKeyField, "DisplayField", displayField));
        }

        return new AttributeCollection(attributes.ToArray());
      }
    }
  }
}