namespace Unleashed.DynamicData
{
  using System;
  using System.Collections.Generic;
  using System.ComponentModel;
  using System.ComponentModel.DataAnnotations;
  using System.Linq;
  using System.Web.DynamicData;
  using System.Web.UI.WebControls;

  public class UnleashedControl : DynamicControl
  {
    private static readonly object FieldValueChangedEvent = new object();

    public new UnleashedFieldTemplate FieldTemplate
    {
      get { return (UnleashedFieldTemplate)base.FieldTemplate; }
    }

    [Browsable(false)]
    public object FieldValue
    {
      get { return this.FieldTemplate.FieldValue; }
      set { this.FieldTemplate.FieldValue = value; }
    }

    public event EventHandler FieldValueChanged
    {
      add 
      { 
        this.Events.AddHandler(FieldValueChangedEvent, value); 
        if (this.FieldTemplate != null)
          this.FieldTemplate.FieldValueChanged += value;
      }
      remove 
      { 
        this.Events.RemoveHandler(FieldValueChangedEvent, value); 
        if (this.FieldTemplate != null)
          this.FieldTemplate.FieldValueChanged -= value;
      }
    }

    protected override void OnInit(EventArgs e)
    {
      // Initialize Mode and ValidationGroup properties based on the parent entity template
      if (this.Mode == DataBoundControlMode.ReadOnly)
      {
        var entityTemplate = this.TemplateControl as EntityTemplateUserControl;
        if (entityTemplate != null)
        {
          this.Mode = entityTemplate.Mode;
          this.ValidationGroup = entityTemplate.ValidationGroup;
        }
      }

      // This method will fire after OnInit event handler in the dynamic entity template, but before the field template is created.
      this.Init += this.InitializeControlParameters;

      // This is where the Column is initialized and the field template created
      base.OnInit(e);

      // Wire up the FieldChangedEvent, now that the field template is available
      var eventHandler = (EventHandler)this.Events[FieldValueChangedEvent];
      if (eventHandler != null)
        this.FieldTemplate.FieldValueChanged += eventHandler;
    }

    private void InitializeControlParameters(object sender, EventArgs args)
    {
      // Column may not have been initialized yet if this control is used from a custom entity template.
      MetaColumn column = this.Column ?? this.Table.GetColumn(this.DataField);

      // Find the UIHintAttribute, if any, and initialize the control parameters
      var uiHint = column.Attributes.OfType<UIHintAttribute>().FirstOrDefault();
      if (uiHint != null)
      {
        foreach (KeyValuePair<string, object> parameter in uiHint.ControlParameters)
          this.SetAttribute(parameter.Key, parameter.Value.ToString());
      }              
    }
  }
}