namespace Unleashed.DynamicData
{
  using System;
  using System.Collections.Generic;
  using System.ComponentModel.DataAnnotations;
  using System.Linq;
  using System.Reflection;
  using System.Web.DynamicData;
  using System.Web.UI;
  using System.Web.UI.WebControls;

  [ViewStateModeById]
  public class UnleashedFilter : Control, IAttributeAccessor, IUnleashedFilterProvider
  {
    private Dictionary<string, string> attributes;
    private UnleashedFilterTemplate filterTemplate;
    private MetaTable table;
    private MetaColumn column;

    public MetaColumn Column
    {
      get
      {
        if (this.column == null)
          this.column = this.Table.GetColumn(this.DataField);
        return this.column;
      }
      set { this.column = value; }
    }

    public string DataField
    {
      get { return (string)this.ViewState["DataField"] ?? string.Empty; }
      set { this.ViewState["DataField"] = value; }
    }

    public IQueryableDataSource DataSource { get; set; }

    public string FilterUIHint
    {
      get { return (string)this.ViewState["FilterUIHint"] ?? string.Empty; }
      set { this.ViewState["FilterUIHint"] = value; }
    }

    public UnleashedFilterTemplate FilterTemplate
    {
      get
      {
        this.EnsureChildControls();
        return this.filterTemplate;
      }
    }

    public MetaTable Table
    {
      get
      {
        if (this.table == null)
          this.table = this.DataSource.GetMetaTable();
        return this.table;
      }
      set { this.table = value; }
    }

    public event EventHandler FilterChanged;

    public string GetAttribute(string key)
    {
      string value;
      if (this.attributes != null && this.attributes.TryGetValue(key, out value))
        return value;
      return string.Empty;
    }

    public IQueryable GetQueryable(IQueryable source)
    {
      return this.FilterTemplate.GetQueryable(source);
    }

    public void SetAttribute(string key, string value)
    {
      if (this.attributes == null)
        this.attributes = new Dictionary<string, string>();
      this.attributes[key] = value;
    }

    protected override void CreateChildControls()
    {
      base.CreateChildControls();
      if (this.filterTemplate == null)
      {
        this.CreateFilterTemplate();
        this.Controls.Add(this.filterTemplate);
      }
    }

    protected virtual void OnFilterChanged(EventArgs args)
    {
      if (this.FilterChanged != null)
        this.FilterChanged(this, args);
    }

    private void CreateFilterTemplate()
    {
      this.filterTemplate = (UnleashedFilterTemplate)this.Table.Model.FilterFactory.CreateFilterControl(this.Column, this.FilterUIHint);
      this.filterTemplate.Initialize(this.Column, this.DataSource, null);
      this.filterTemplate.FilterChanged += (sender, args) => this.OnFilterChanged(args);

      var hint = this.Column.Attributes.OfType<FilterUIHintAttribute>().FirstOrDefault();
      if (hint != null)
      {
        foreach (KeyValuePair<string, object> parameter in hint.ControlParameters)
        {
          if (string.IsNullOrEmpty(this.GetAttribute(parameter.Key)))
            this.SetAttribute(parameter.Key, parameter.Value.ToString());
        }
      }              

      if (this.attributes != null)
      {
        var type = this.filterTemplate.GetType();
        foreach (KeyValuePair<string, string> entry in this.attributes)
        {
          PropertyInfo propInfo = type.GetProperty(entry.Key, BindingFlags.Public | BindingFlags.Instance | BindingFlags.IgnoreCase);
          if (propInfo != null)
          {
            var value = PropertyConverter.ObjectFromString(propInfo.PropertyType, propInfo, entry.Value);
            propInfo.SetValue(this.filterTemplate, value, null);
          }
        }
      }
    }
  }
}