namespace Unleashed.DynamicData
{
  using System;
  using System.Collections.Generic;
  using System.ComponentModel;
  using System.Linq;
  using System.Web.DynamicData;
  using System.Web.UI;
  using System.Web.UI.WebControls;

  [ViewStateModeById] // DO WE NEED THIS?
  [ParseChildren(true)]
  [PersistChildren(false)]
  public class UnleashedFilterRepeater : Control, IUnleashedFilterProvider
  {
    private MetaTable table;
    private List<UnleashedFilter> filters = new List<UnleashedFilter>();

    public IQueryableDataSource DataSource { get; set; }

    [DefaultValue("DynamicFilter")]
    public string DynamicFilterContainerId
    {
      get { return (string)this.ViewState["DynamicFilterContainerId"] ?? "DynamicFilter"; }
      set { this.ViewState["DynamicFilterContainerId"] = value; }
    }

    [Browsable(false)]
    [DefaultValue(null)]
    [PersistenceMode(PersistenceMode.InnerProperty)]
    [TemplateContainer(typeof(INamingContainer))]
    public virtual ITemplate ItemTemplate { get; set; }

    public MetaTable Table
    {
      get
      {
        if (this.table == null)
          this.table = this.DataSource.GetMetaTable();
        return this.table;
      }
      set { this.table = value; }
    }

    public IQueryable GetQueryable(IQueryable source)
    {
      foreach (UnleashedFilter filter in this.filters)
        source = filter.GetQueryable(source);
      return source;
    }

    protected override void CreateChildControls()
    {
      base.CreateChildControls();
      int itemIndex = 0;
      foreach (MetaColumn column in this.Table.GetFilteredColumns())
      {
        var container = new DataItemContainer
        {
          ID = "__" + column.Name + "Filter", // DO WE NEED THIS?
          DataItem = column,
          DataItemIndex = itemIndex, 
          DisplayIndex = itemIndex
        };
        this.ItemTemplate.InstantiateIn(container);
        this.Controls.Add(container);

        var filter = (UnleashedFilter)container.FindControl(this.DynamicFilterContainerId);
        if (filter == null)
          throw new InvalidOperationException("Could not find control in template");
        filter.DataSource = this.DataSource;
        filter.DataField = column.Name;
        filter.Column = column;

        container.DataBind();
        container.DataItem = null; // WHY??

        this.filters.Add(filter);
        itemIndex++;
      }
    }
  }
}