namespace Unleashed.EntityFramework
{
  using System;
  using System.Collections;
  using System.Collections.Generic;
  using System.ComponentModel;
  using System.Data.Objects;
  using System.Linq;
  using System.Linq.Expressions;
  using DataAnnotations;

  public class FilteredObjectSet<T> : IObjectSet<T> where T : class
  {
    private static readonly CustomQueryAttribute CustomQueryAttribute = TypeDescriptor.GetAttributes(typeof(T)).OfType<CustomQueryAttribute>().FirstOrDefault();
    private readonly ObjectSet<T> original;

    public FilteredObjectSet(ObjectSet<T> original)
    {
      this.original = original;
    }

    public IEnumerator<T> GetEnumerator()
    {
      return ((IEnumerable<T>)this.original).GetEnumerator();
    }

    IEnumerator IEnumerable.GetEnumerator()
    {
      return this.GetEnumerator();
    }

    public Expression Expression
    {
      get { return this.GetQueryable().Expression; }
    }

    public Type ElementType
    {
      get { return this.GetQueryable().ElementType; }
    }

    public IQueryProvider Provider
    {
      get { return this.GetQueryable().Provider; }
    }

    public void AddObject(T entity)
    {
      this.original.AddObject(entity);
    }

    public void Attach(T entity)
    {
      this.original.Attach(entity);
    }

    public void DeleteObject(T entity)
    {
      this.original.DeleteObject(entity);
    }

    public void Detach(T entity)
    {
      this.original.Detach(entity);
    }

    private IQueryable GetQueryable()
    {
      if (CustomQueryAttribute != null)
        return CustomQueryAttribute.GetQueryable(this.original);
      return this.original;
    }
  }
}
