﻿using System;
using System.Collections.Specialized;
using System.ComponentModel.DataAnnotations;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Unleashed.DynamicData;

namespace WebApplication.DynamicData.FieldTemplates
{
  public partial class DateTime_EditField : UnleashedFieldTemplate
  {
    private static readonly DataTypeAttribute DefaultDateAttribute = new DataTypeAttribute(DataType.DateTime);

    public override Control DataControl
    {
      get { return this.textBox; }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
      this.textBox.ToolTip = this.Column.Description;

      this.SetUpValidator(this.requiredFieldValidator);
      this.SetUpValidator(this.regularExpressionValidator);
      this.SetUpValidator(this.dynamicValidator);
      this.SetUpCustomValidator(this.dateValidator);
    }

    protected override void ExtractValues(IOrderedDictionary dictionary)
    {
      dictionary[this.Column.Name] = this.ConvertEditedValue(this.textBox.Text);
    }

    protected void DateValidator_ServerValidate(object source, ServerValidateEventArgs args)
    {
      DateTime dummyResult;
      args.IsValid = DateTime.TryParse(args.Value, out dummyResult);
    }

    private void SetUpCustomValidator(CustomValidator validator)
    {
      if (this.Column.DataTypeAttribute != null)
      {
        switch (this.Column.DataTypeAttribute.DataType)
        {
          case DataType.Date:
          case DataType.DateTime:
          case DataType.Time:
            this.EnableCustomValidator(validator, this.Column.DataTypeAttribute);
            break;
        }
      }
      else if (this.Column.ColumnType.Equals(typeof(DateTime)))
      {
        this.EnableCustomValidator(validator, DefaultDateAttribute);
      }
    }

    private void EnableCustomValidator(CustomValidator validator, DataTypeAttribute attribute)
    {
      validator.Enabled = true;
      validator.ErrorMessage = HttpUtility.HtmlEncode(attribute.FormatErrorMessage(this.Column.DisplayName));
    }
  }
}