﻿using System;
using System.Collections.Specialized;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Unleashed.DynamicData;

namespace WebApplication.DynamicData.FieldTemplates
{
  public partial class PhoneNumberEditField : UnleashedFieldTemplate
  {
    public override Control DataControl
    {
      get { return this.textBox; }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
      this.textBox.ToolTip = this.Column.Description;

      this.SetUpValidator(this.requiredFieldValidator);
      this.SetUpValidator(this.dynamicValidator);
      this.SetUpRegexValidator(this.regularExpressionValidator);
    }

    protected override void ExtractValues(IOrderedDictionary dictionary)
    {
      dictionary[this.Column.Name] = this.ConvertEditedValue(this.textBox.Text);
    }

    private void SetUpRegexValidator(RegularExpressionValidator validator)
    {
      RegularExpressionAttribute attribute = this.Column.Attributes.OfType<RegularExpressionAttribute>().FirstOrDefault();
      if (attribute == null)
      {
        attribute = new RegularExpressionAttribute(@"\(\d{3}\)\d{3}-\d{4}");
        attribute.ErrorMessage = "The field {0} must be a valid phone number.";
      }

      validator.Enabled = true;
      validator.Text = "*";
      validator.ValidationExpression = attribute.Pattern;
      validator.ErrorMessage = HttpUtility.HtmlEncode(attribute.FormatErrorMessage(this.Column.DisplayName));
      validator.ToolTip = validator.ErrorMessage;

      this.IgnoreModelValidationAttribute(attribute.GetType());
    }
  }
}