﻿using System;
using System.Linq;
using System.Linq.Dynamic;
using System.Web.UI;
using Unleashed.DynamicData;

namespace WebApplication.DynamicData.Filters
{
  public partial class DateFilter : UnleashedFilterTemplate
  {
    private const string errorMessage = "{0} value must be a valid date.";

    public override Control FilterControl
    {
      get { return this.textBox; }
    }

    protected override void OnInit(EventArgs e)
    {
      base.OnInit(e);
      this.dateValidator.ErrorMessage = string.Format(errorMessage, this.Column.Name);
      this.dateValidator.ToolTip = this.dateValidator.ErrorMessage;
    }

    protected void TextBox_TextChanged(object sender, EventArgs e)
    {
      this.OnFilterChanged();
    }

    public override IQueryable GetQueryable(IQueryable source)
    {
      if (!string.IsNullOrEmpty(this.textBox.Text))
      {
        this.dateValidator.Validate();
        if (this.dateValidator.IsValid)
        {
          DateTime date = DateTime.Parse(this.textBox.Text);
          var startDate = new DateTime(date.Year, date.Month, date.Day);
          DateTime endDate = startDate.AddDays(1);
          string filter = string.Format("{0} >= @0 && {0} < @1", this.Column.Name);
          source = source.Where(filter, startDate, endDate);
        }
      }

      return source;
    }
  }
}