﻿using System;
using System.Linq;
using System.Linq.Dynamic;
using System.Web.UI;
using Unleashed.DynamicData;

namespace WebApplication.DynamicData.Filters
{
  public partial class TextFilter : UnleashedFilterTemplate
  {
    public override Control FilterControl
    {
      get { return this.textBox; }
    }

    public int Columns { get; set; }

    public override IQueryable GetQueryable(IQueryable query)
    {
      if (!string.IsNullOrEmpty(this.textBox.Text))
      {
        try
        {
          object value = Convert.ChangeType(this.textBox.Text, this.Column.ColumnType);
          string filter = this.Column.Name + " == @0";
          query = query.Where(filter, value);
        }
        catch (FormatException)
        {
          this.validator.IsValid = false;
          this.validator.ErrorMessage = string.Format(
            "'{0}' is not a valid filter value for {1}",
            this.textBox.Text, this.Column.DisplayName);
          this.validator.ToolTip = this.validator.ErrorMessage;
        }
      }

      return query;
    }

    protected override void OnInit(EventArgs e)
    {
      base.OnInit(e);
      this.textBox.MaxLength = this.Column.MaxLength;
      this.textBox.Text = this.DefaultValue;
      if (this.Columns > 0)
        this.textBox.Columns = this.Columns;
    }

    protected void TextBox_TextChanged(object sender, EventArgs e)
    {
      this.OnFilterChanged();
    }
  }
}