﻿using System;
using System.Collections.Generic;
using System.Web.DynamicData;
using System.Web.Routing;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using Unleashed.DynamicData;

namespace WebApplication.DynamicData.PageTemplates
{
  public partial class List : Page
  {
    protected UnleashedMetaTable table;
    private bool canUserDelete;
    private bool canUserInsert;
    private bool canUserUpdate;

    protected override void OnPreInit(EventArgs e)
    {
      base.OnPreInit(e);
      this.table = (UnleashedMetaTable)DynamicDataRouteHandler.GetRequestMetaTable(this.Context);
      if (!this.table.CanRead(this.User))
      {
        FormsAuthentication.RedirectToLoginPage();
        this.Response.End();
      }

      this.canUserDelete = this.table.CanDelete(this.User);
      this.canUserInsert = this.table.CanInsert(this.User);
      this.canUserUpdate = this.table.CanUpdate(this.User);
    }

    protected void Page_Init(object sender, EventArgs e)
    {
      this.Title = this.table.DisplayName;

      this.dataSource.EntityTypeFilter = this.table.EntityType.Name;
      this.dataSource.Include = this.table.ForeignKeyColumnsNames;

      IDictionary<string, object> columnValues = this.table.GetColumnValuesFromRoute(this.Context);
      this.gridView.SetMetaTable(this.table, columnValues);

      var routeValues = new RouteValueDictionary(columnValues);
      this.insertLink.NavigateUrl = this.table.GetActionPath(PageAction.Insert, routeValues);
    }

    protected void Page_InitComplete(object sender, EventArgs e)
    {
      this.gridView.ColumnsGenerator = new UnleashedFieldGenerator(this.table);
    }

    protected void Label_PreRender(object sender, EventArgs e)
    {
      var label = (Label)sender;
      var dynamicFilter = (UnleashedFilter)label.FindControl("DynamicFilter");
      var template = dynamicFilter.FilterTemplate as QueryableFilterUserControl;
      if (template != null && template.FilterControl != null)
        label.AssociatedControlID = template.FilterControl.GetUniqueIDRelativeTo(label);
    }

    protected void DynamicFilter_FilterChanged(object sender, EventArgs e)
    {
      this.gridView.PageIndex = 0;
    }

    protected void EditLink_Init(object sender, EventArgs e)
    {
      ((DynamicHyperLink)sender).Visible = this.canUserUpdate;
    }

    protected void DeleteLink_Init(object sender, EventArgs e)
    {
      ((DynamicHyperLink)sender).Visible = this.canUserDelete;
    }

    protected void InsertLink_Init(object sender, EventArgs e)
    {
      ((DynamicHyperLink)sender).Visible = this.canUserInsert;
    }

    protected void DataSource_QueryCreated(object sender, QueryCreatedEventArgs e)
    {
      e.Query = this.table.GetQueryable(e.Query);
    }
  }
}