﻿using System;
using System.Web.DynamicData;
using System.Web.UI;
using System.Web.UI.WebControls;
using Unleashed.DynamicData;

namespace WebApplication.DynamicData.PageTemplates
{
  public partial class ListDetails : Page
  {
    protected MetaTable table;

    protected void Page_Init(object sender, EventArgs e)
    {
      this.table = DynamicDataRouteHandler.GetRequestMetaTable(this.Context);
      this.Title = this.table.DisplayName;

      this.gridView.SetMetaTable(this.table, this.table.GetColumnValuesFromRoute(this.Context));
      this.gridDataSource.EntityTypeFilter = this.table.EntityType.Name;
      this.gridDataSource.Include = this.table.ForeignKeyColumnsNames;

      this.formView.SetMetaTable(this.table);
      this.formDataSource.EntityTypeFilter = this.table.EntityType.Name;
      this.formDataSource.Include = this.table.ForeignKeyColumnsNames;

      // Selection from url
      if (!this.Page.IsPostBack && this.table.HasPrimaryKey)
      {
        this.gridView.SelectedPersistedDataKey = this.table.GetDataKeyFromRoute();
        if (this.gridView.SelectedPersistedDataKey == null)
          this.gridView.SelectedIndex = 0;
      }

      // Disable various options if the table is readonly
      if (this.table.IsReadOnly)
      {
        this.DetailsPanel.Visible = false;
        this.gridView.AutoGenerateSelectButton = false;
        this.gridView.AutoGenerateEditButton = false;
        this.gridView.AutoGenerateDeleteButton = false;
        this.gridView.EnablePersistedSelection = false;
      }
    }

    protected void Page_InitComplete(object sender, EventArgs e)
    {
      this.gridView.ColumnsGenerator = new UnleashedFieldGenerator(this.table);
    }

    protected void GridView_DataBound(object sender, EventArgs e)
    {
      if (this.gridView.Rows.Count == 0)
        this.formView.ChangeMode(FormViewMode.Insert);
    }

    protected void Label_PreRender(object sender, EventArgs e)
    {
      var label = (Label)sender;
      var dynamicFilter = (UnleashedFilter)label.FindControl("DynamicFilter");
      UnleashedFilterTemplate template = dynamicFilter.FilterTemplate;
      if (template != null && template.FilterControl != null)
        label.AssociatedControlID = template.FilterControl.GetUniqueIDRelativeTo(label);
    }

    protected void DynamicFilter_FilterChanged(object sender, EventArgs e)
    {
      this.gridView.EditIndex = -1;
      this.gridView.PageIndex = 0;
      this.formView.ChangeMode(FormViewMode.ReadOnly);
    }

    protected void GridView_RowEditing(object sender, EventArgs e)
    {
      this.formView.ChangeMode(FormViewMode.ReadOnly);
    }

    protected void GridView_SelectedIndexChanging(object sender, EventArgs e)
    {
      this.gridView.EditIndex = -1;
      this.formView.ChangeMode(FormViewMode.ReadOnly);
    }

    protected void GridView_RowCreated(object sender, GridViewRowEventArgs e)
    {
      this.SetDeleteConfirmation(e.Row);
    }

    protected void DataBindFormView(object sender, EventArgs e)
    {
      this.formView.DataBind();
    }

    protected void DataBindGridView(object sender, EventArgs e)
    {
      this.gridView.DataBind();
    }

    protected void FormView_ModeChanging(object sender, FormViewModeEventArgs e)
    {
      if (e.NewMode != FormViewMode.ReadOnly)
        this.gridView.EditIndex = -1;
    }

    protected void FormView_PreRender(object sender, EventArgs e)
    {
      if (this.formView.Row != null)
        this.SetDeleteConfirmation(this.formView.Row);
    }

    private void SetDeleteConfirmation(TableRow row)
    {
      foreach (Control c in row.Cells[0].Controls)
      {
        var button = c as LinkButton;
        if (button != null && button.CommandName == DataControlCommands.DeleteCommandName)
          button.OnClientClick = "return confirm('Are you sure you want to delete this item?');";
      }
    }
  }
}