﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Web.DynamicData;
using System.Web.UI.WebControls;

namespace WebApplication.MetaDataExplorer
{
  public static class MetaDataExplorer
  {
    public static void DisplayObjectDetails(object obj, DetailsView view)
    {
      view.HeaderText = obj.GetType().FullName;
      view.AutoGenerateRows = false;
      view.Fields.Clear();

      IEnumerable<PropertyInfo> properties = obj.GetType().GetProperties().Cast<PropertyInfo>().OrderBy(property => property.Name).ToList();
      foreach (PropertyInfo property in properties)
      {
        view.Fields.Add(CreateField(property, obj));
      }

      view.DataSource = new[] { obj };
      view.DataBind();
    }

    private static DataControlField CreateField(PropertyInfo property, object obj)
    {
      if (typeof(Attribute).IsAssignableFrom(property.PropertyType))
      {
        Attribute attribute = (Attribute)property.GetValue(obj, null);
        if (attribute != null)
        {
          return new ButtonField()
          {
            HeaderText = property.Name,
            CommandName = property.Name,
            Text = "(...)",
            ShowHeader = true
          };
        }
      }
      else if (typeof(MetaModel).IsAssignableFrom(property.PropertyType))
      {
          return new HyperLinkField()
          {
            HeaderText = property.Name,
            NavigateUrl = "MetaModel.aspx",
            Text = "(...)"
          };
      }
      else if (typeof(MetaTable).IsAssignableFrom(property.PropertyType))
      {
        MetaTable table = (MetaTable)property.GetValue(obj, null);
        return new HyperLinkField()
        {
          HeaderText = property.Name,
          NavigateUrl = "MetaTable.aspx?Name=" + table.Name,
          Text = table.Name
        };
      }
      else if (typeof(MetaColumn).IsAssignableFrom(property.PropertyType))
      {
        MetaColumn column = (MetaColumn)property.GetValue(obj, null);
        if (column != null)
        {
          return new HyperLinkField()
          {
            HeaderText = property.Name,
            NavigateUrl = "MetaColumn.aspx?Table=" + column.Table.Name + "&Column=" + column.Name,
            Text = column.Name
          };
        }
      }
      else if (typeof(AttributeCollection).IsAssignableFrom(property.PropertyType) 
           || typeof(IEnumerable<MetaColumn>).IsAssignableFrom(property.PropertyType)
           || typeof(IEnumerable<MetaTable>).IsAssignableFrom(property.PropertyType))
      {
        return new ButtonField()
        {
          HeaderText = property.Name,
          Text = "(...)",
          CommandName = property.Name, 
          ShowHeader = true
        };
      }
      else if (typeof(IEnumerable).IsAssignableFrom(property.PropertyType) 
           && property.PropertyType != typeof(string))
      {
        return new EnumerableField()
        {
          DataField = property.Name,
          HeaderText = property.Name
        };
      }

      return new ExceptionResistantField()
      {
        DataField = property.Name,
        HeaderText = property.Name
      };
    }
  }
}