﻿using System;
using System.Linq;
using System.Reflection;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.DynamicData;

namespace WebApplication.MetaDataExplorer
{
  public partial class MetaTablePage : Page
  {
    protected MetaTable table;

    protected override void  OnInit(EventArgs e)
    {
 	    base.OnInit(e);
      string tableName = this.Request.QueryString["Name"];
      this.table = MetaModel.Default.GetTable(tableName);
    }

    protected void Page_Load(object sender, EventArgs e)
    {
      if (!this.IsPostBack)
      {
        MetaDataExplorer.DisplayObjectDetails(this.table, this.tableDetails);

        this.gridView.Columns.Add(this.CreateHyperLinkField());
        this.gridView.Columns[0].HeaderText = "Columns";
        this.gridView.DataSource = this.table.Columns;
        this.gridView.DataBind();
      }
    }

    protected void TableDetails_ItemCommand(object sender, System.Web.UI.WebControls.DetailsViewCommandEventArgs e)
    {
      this.gridView.Columns.Clear();

      switch (e.CommandName)
      {
        case "Attributes":
          this.gridView.DataSource = table.Attributes.Cast<Attribute>().Select(a => a.GetType());
          this.gridView.Columns.Add(new ButtonField() { CommandName="Attribute", DataTextField = "Name", DataTextFormatString="{0}",  });
          break;

        case "Columns":
          this.gridView.DataSource = table.Columns;
          this.gridView.Columns.Add(this.CreateHyperLinkField());
          break;

        case "PrimaryKeyColumns":
          this.gridView.DataSource = table.PrimaryKeyColumns;
          this.gridView.Columns.Add(this.CreateHyperLinkField());
          break;
      }

      this.gridView.Columns[0].HeaderText = e.CommandName;
      this.gridView.DataBind();

      this.attributeDetails.Visible = (e.CommandName == "Attributes");
    }

    protected void GridView_RowCommand(object sender, GridViewCommandEventArgs e)
    {
      if (e.CommandName == "Attribute")
      {
        int attributeIndex = int.Parse((string)e.CommandArgument);
        Attribute attribute = this.table.Attributes[attributeIndex];
        MetaDataExplorer.DisplayObjectDetails(attribute, this.attributeDetails);
      }
    }

    private HyperLinkField CreateHyperLinkField()
    {
      HyperLinkField field = new HyperLinkField();
      field.DataTextField = "Name";
      field.DataNavigateUrlFormatString = "MetaColumn.aspx?Table=" + this.table.Name + "&Column={0}";
      field.DataNavigateUrlFields = new string [] { "Name" };
      return field;
    }
  }
}