﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;

namespace WebApplication
{
  [ServiceContract]
  public class RegionCompletionService
  {
    private static readonly Dictionary<string, string[]> regionsByCountry = 
      new Dictionary<string, string[]>(StringComparer.InvariantCultureIgnoreCase)
      {
        { "Canada", 
          new[] {
            "ON", "QC", "NS", "NB", "MB", "BC", "PE", "SK", "AB", "NL", 
            "NT", "UT", "NU"
          }
        },
        { "USA", 
          new[] {
            "AL", "AK", "AZ", "AR", "CA", "CO", "CT", "DE", "FL", "GA",
            "HI", "ID", "IL", "IN", "IA", "KS", "KY", "LA", "ME", "MD",
            "MA", "MI", "MN", "MS", "MO", "MT", "NE", "NV", "NH", "NJ",
            "NM", "NY", "NC", "ND", "OH", "OK", "OR", "PA", "RI", "SC",
            "SD", "TN", "TX", "UT", "VT", "VA", "WA", "WV", "WI", "WY"
          }
        }
      };

    [OperationContract]
    public string[] GetCompletionList(string prefixText, int count, string contextKey)
    {
      var country = contextKey ?? string.Empty;
      string[] regions;
      if (regionsByCountry.TryGetValue(country, out regions))
      {
        var ignoreCase = StringComparison.InvariantCultureIgnoreCase;
        return regions
          .Where(r => r.StartsWith(prefixText, ignoreCase))
          .Take(count)
          .ToArray();
      }
      return new string[0];
    }
  }
}
